@@ -56,7 +56,9 @@ dupfilter_time <- function (sdata, step.time = 0, no.cores = 1) {
 
   
   #### Get step time
-  sdata <- track_param(sdata, param = c('time', 'distance'))
+  if(nrow(sdata) > 1){
+    sdata <- track_param(sdata, param = c('time', 'distance'))
+  }
   
   
   #### Filter successive locations with the exactly same coordinates
@@ -68,7 +70,9 @@ dupfilter_time <- function (sdata, step.time = 0, no.cores = 1) {
       sdata2 <- dplyr::anti_join(sdata, sdata1, by = c('id', 'DateTime', 'lat', 'lon', 'qi'))
 
       #### Group temporal duplicates
-      sdata1 <- track_param(sdata1, param = 'distance')
+      if(nrow(sdata1) > 1){
+        sdata1 <- track_param(sdata1, param = 'distance')
+      }
       index <- 0; g <- rep(0, nrow(sdata1))
       for(i in 1:nrow(sdata1)){
         if(any(is.na(sdata1[i, 'pDist']) | sdata1[i, 'sDist'] == 0, na.rm = TRUE)){
@@ -87,7 +91,9 @@ dupfilter_time <- function (sdata, step.time = 0, no.cores = 1) {
       rm(sdata1, sdata2)
 
       #### Recalculate movement parameters
-      sdata <- track_param(sdata, param = c('time', 'distance'))
+      if(nrow(sdata) > 1){
+        sdata <- track_param(sdata, param = c('time', 'distance'))
+      }
   }
 
 
@@ -103,7 +109,9 @@ dupfilter_time <- function (sdata, step.time = 0, no.cores = 1) {
     
     
     #### Group temporal duplicates
-    sdata1 <- track_param(sdata1, param = 'time')
+    if(nrow(sdata1) > 1){
+      sdata1 <- track_param(sdata1, param = 'time')
+    }
     index <- 0; g <- rep(0, nrow(sdata1))
     for(i in 1:nrow(sdata1)){
       if(any(is.na(sdata1[i, 'pTime']) | (sdata1[i, 'sTime'] <= step.time), na.rm = TRUE)){
@@ -199,11 +207,13 @@ dupfilter_time <- function (sdata, step.time = 0, no.cores = 1) {
     
     #### Combine
     sdata <- dplyr::bind_rows(sdata1, sdata2, sdata3)
-    sdata$group <- NULL
+    # sdata$group <- NULL
     rm(sdata1, sdata2, sdata3)
     
     ## Re-calculate
-    sdata <- track_param(sdata, param = 'time')
+    if(nrow(sdata) > 1){
+      sdata <- track_param(sdata, param = 'time')
+    }
 
     #### Return
     return(sdata)
@@ -250,7 +260,7 @@ dupfilter_time <- function (sdata, step.time = 0, no.cores = 1) {
 
   
   #### Delete working columns and return the output
-  drop.vars <- c("pSpeed", "sSpeed", "inAng", "meanSpeed", "meanAngle")
+  drop.vars <- c("pSpeed", "sSpeed", "inAng", "meanSpeed", "meanAngle", 'group')
   sdata <- sdata[,!(names(sdata) %in% drop.vars)] 
   return(sdata)
 }
